"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.binder = exports.formBinderURL = void 0;
const rxjs_1 = require("rxjs");
const mybinderURL = "https://mybinder.org";
/**
 * Strips the github.com URL base from a repo URL.
 *
 * @param repo A complete repo URL
 *
 * @returns The repo without github.com
 */
const cleanRepo = (repo) => repo
    .replace(/^(https?:\/\/)?github.com\//, "")
    // trim trailing or leading '/' on repo
    .replace(/(^\/)|(\/?$)/g, "");
/**
 * Returns a URL for initializing a connection to Binder.
 *
 * @param options An object containing the details of the Binder connection
 *
 * @returns A Binder URL
 */
exports.formBinderURL = ({ repo = "jupyter/notebook", ref = "master", binderURL = mybinderURL } = {}) => {
    repo = cleanRepo(repo);
    binderURL = binderURL.replace(/(\/?$)/g, ""); // trim trailing / on url
    return `${binderURL}/build/gh/${repo}/${ref}`;
};
const defaultEventSource = typeof window !== "undefined" &&
    "EventSource" in window &&
    window.EventSource;
/**
 * Returns an observable stream for mybinder.org or any related
 *
 * @param options repo and other options to connect
 * @param EventSourceDI allows overriding EventSource for testing
 *
 * @returns An Observable with a connection to a Binder instance
 */
exports.binder = (options, EventSourceDI = defaultEventSource) => {
    if (!EventSourceDI) {
        throw new Error("Event Source not supported on this platform -- please polyfill");
    }
    const url = exports.formBinderURL(options);
    return rxjs_1.Observable.create((observer) => {
        const es = new EventSourceDI(url);
        es.onmessage = (evt) => {
            const msg = JSON.parse(evt.data);
            // Pass messages onward, closing on "failed" or "ready"
            observer.next(msg);
            switch (msg.phase) {
                case "failed":
                    // The message of failed is a message and shouldn't go on the error
                    // part of the stream, we should just complete it
                    observer.complete();
                    break;
                case "ready":
                    // When the server is ready, we can close the event source
                    observer.complete();
                    break;
                default:
                    // do nothing, we already sent the message on
                    break;
            }
        };
        es.onerror = err => {
            observer.error(err);
        };
        // disposal of the observable closes the EventSource
        return () => {
            es.close();
        };
    });
};
//# sourceMappingURL=index.js.map